# -*- coding: utf-8 -*-
"""
dame.

:license: Apache 2.0
"""
import logging
from pathlib import Path

import click
import semver
from modelcatalog import ApiException, Configuration

import dame
from dame import _utils
from dame.cli_methods import verify_input_parameters, run_method_setup, show_model_configuration_details, \
    print_table_list, edit_parameters, show_model_configuration_details_dt
from dame.configuration import configure_credentials, get_credentials, DEFAULT_PROFILE
from dame.modelcatalogapi import get_setup, get_model_configuration, list_model_configuration, list_setup, \
    get_data_transformation, get_transformation_dataset
from urllib.parse import urlparse

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader


@click.group()
@click.option("--verbose", "-v", default=0, count=True)
def cli(verbose):
    _utils.init_logger()
    lv = ".".join(_utils.get_latest_version().split(".")[:3])
    cv = ".".join(dame.__version__.split(".")[:3])

    if semver.compare(lv, cv) > 0:
        click.secho(
            f"""WARNING: You are using dame-cli version {dame.__version__}, however version {lv} is available.
You should consider upgrading via the 'pip install --upgrade dame-cli' command.""",
            fg="yellow",
        )


@cli.command(help="Configure credentials")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
@click.option('--server', prompt='Model Catalog API',
              help='The Model Catalog API', required=True, default=Configuration().host, show_default=True)
@click.option('--username', prompt='Username',
              help='Your email.', required=True, default="mint@isi.edu", show_default=True)
def credentials(server, username, profile="default"):
    try:
        configure_credentials(server, username, profile)
    except Exception:
        click.secho(f"Failed", fg="red")
    click.secho(f"Success", fg="green")


@cli.command(help="Show dame-cli version.")
def version():
    click.echo(f"DAME: v{dame.__version__}")


@cli.command(help="Open the Model Catalog in your browser")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
def browse(profile):
    credentials = get_credentials(profile)
    server = ''
    if "server" in credentials:
        click.secho(f"""Looking User Interface {credentials['server']}""")
        server = credentials['server']
    else:
        click.secho("Unable to find the model catalog", fg="red")
    domain = urlparse(server).netloc
    mapping = {
        "api.models.mint.isi.edu": "https://models.mint.isi.edu",
        "api.models.wildfire.mint.isi.edu" : "https://wildfire.models.mint.isi.edu"
    }
    if not domain or domain not in mapping:
        click.secho("Unable to find the model catalog", fg="red")

    click.launch(mapping[domain])


@cli.command(help="Run a model configuration or model configuration setup")
@click.argument(
    "name",
    type=click.STRING
)
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    metavar="<profile-name>",
    default="default",
)
@click.option(
    "--data",
    "-d",
    type=click.Path(exists=False, dir_okay=True, resolve_path=True),
)
@click.option('--interactive/--non-interactive', default=True)
def run(name, interactive, profile, data):
    if not data or not Path(data).exists():
        data = None
    else:
        data = Path(data)

    try:
        config = get_model_configuration(name, profile=profile)
    except ApiException as e:
        click.secho("{}".format(e.reason))
        exit(0)
    click.clear()
    if "ModelConfigurationSetup" in config.type:
        resource = get_setup(name, profile=profile)
    elif "ModelConfiguration" in config.type or "DataTransformation":
        resource = get_model_configuration(name, profile=profile)
    try:
        show_model_configuration_details(resource)
    except AttributeError as e:
        click.secho("Unable to run it: {}".format(str(e)), fg="red")
        exit(1)


    try:
        verify_input_parameters(resource, interactive, data, profile)
    except ValueError as e:
        click.secho("Unable to run. Please use interactive mode", fg="yellow")
        exit(1)
    try:
        if interactive and click.confirm("Do you want to edit the parameters?"):
            edit_parameters(resource, interactive)
    except ValueError as e:
        click.secho("Unable to run. Please use interactive mode", fg="yellow")
        exit(1)
    run_method_setup(resource, interactive, data)


"""
Run a modelconfiguration or modelconfiguration
"""


@cli.group()
def model_configuration():
    """Manages model configurations"""


@model_configuration.command(name="list", help="List configurations")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default=DEFAULT_PROFILE,
    metavar="<profile-name>",
)
def model_configuration_list(profile):
    items = list_model_configuration(label=None, profile=profile)
    print_table_list(items)


@click.argument(
    "name",
    type=click.STRING
)
@model_configuration.command(name="show", help="Show model configuration")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default=DEFAULT_PROFILE,
    metavar="<profile-name>",
)
def model_configuration_show(name, profile):
    try:
        _setup = get_model_configuration(name, profile=profile)
    except ApiException as e:
        click.secho("{}".format(e.reason))
        exit(1)
    try:
        show_model_configuration_details(_setup)
    except AttributeError as e:
        click.secho("This setup is not executable.\n".format(e), fg="red")
        exit(1)

@cli.group()
def setup():
    """Manages model configuration setup"""


@setup.command(name="list", help="List model configuration setups")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default=DEFAULT_PROFILE,
    metavar="<profile-name>",
)
def setup_list(profile):
    items = list_setup(label=None, profile=profile)
    print_table_list(items)


@click.argument(
    "name",
    type=click.STRING
)
@setup.command(name="show", help="Show model configuration setups")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default=DEFAULT_PROFILE,
    metavar="<profile-name>",
)
def setup_show(name, profile):
    try:
        _setup = get_setup(name, profile=profile)
    except ApiException as e:
        click.secho("{}".format(e.reason))
        exit(1)
    try:
        show_model_configuration_details(_setup)
    except AttributeError as e:
        click.secho("This setup is not executable.\n".format(e), fg="red")


@setup.command(name="list", help="List model configuration setups")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default=DEFAULT_PROFILE,
    metavar="<profile-name>",
)
def setup_list(profile):
    items = list_setup(label=None, profile=profile)
    print_table_list(items)


@cli.group()
def transformation():
    """Manages Data transformation"""


@transformation.command(name="list")
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
@click.argument(
    "dataset_id",
    type=click.STRING,
    required=False
)
def transformation_list(dataset_id, profile):
    """
    List the transformations available. For example:

    $ dame transformation list

    You can see the transformation available for a DataSetSpecification using

    $ dame transformation list topoflow36_2.1.0_rainRates
    """
    if dataset_id:
        items = get_transformation_dataset(dataset_id, profile=profile)
    else:
        items = get_data_transformation(profile=profile)
    print_table_list(items)


@transformation.command(name="run")
@click.argument(
    "data_transformation_id",
    type=click.STRING
)
@click.option(
    "--profile",
    "-p",
    envvar="MINT_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
def transformation_run(data_transformation_id, profile):
    """
    You must pass the argument ID (ID of the transformation)

    For example:

    dame transformation run topoflow_climate
    """
    resource = get_model_configuration(data_transformation_id, profile=profile)
    interactive = True
    data = Path('.')
    try:
        show_model_configuration_details(resource)
    except AttributeError as e:
        click.secho("This setup is not executable.\n".format(e), fg="red")
    run_method_setup(resource, interactive, data)



# -*- coding: utf-8 -*-
import logging.config
import requests

log = logging.getLogger()


def init_logger():
    LOGLEVEL = logging.DEBUG
    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'default': {
                # exact format is not important, this is the minimum information
                'format': '%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
            },
        },
        'handlers': {
            # console logs to stderr
            'console': {
                'class': 'logging.StreamHandler',
                'formatter': 'default',
            },
        },
        'loggers': {
            # default for all undefined Python modules
            '': {
                'level': 'WARNING',
                'handlers': ['console'],
            },
            # Our application code
            'wcm': {
                'level': LOGLEVEL,
                'handlers': ['console'],
                # Avoid double logging because of root logger
                'propagate': False,
            },
            # Prevent noisy modules from logging to Sentry
            'noisy_module': {
                'level': 'ERROR',
                'handlers': ['console'],
                'propagate': False,
            },
        },
    })
    logger = logging.getLogger(__package__)


def get_latest_version():
    return requests.get("https://pypi.org/pypi/dame-cli/json").json()["info"]["version"]



import http
import json

import urllib3
from dame.utils import check_is_none
import certifi

http = urllib3.PoolManager(
    cert_reqs='CERT_REQUIRED',  # Force certificate check.
    ca_certs=certifi.where(),  # Path to the Certifi bundle.
)
chunk_size = 65536


def check_size(url):
    r = http.request('GET', url, preload_content=False)
    content_bytes = r.headers.get("Content-Length")
    r.release_conn()
    return content_bytes


def parse_inputs(model, thread):
    inputs = []
    for input in thread.models[model]['input_files']:
        available_resources = check_is_none(check_is_none(input, 'value'), 'resources')
        for r in available_resources:
            if check_is_none(r, "selected"):
                download_url = check_is_none(r, 'url')
                input["download_url"] = download_url
        inputs.append(input)
    return inputs


def parse_outputs(model, thread, results):
    outputs = []
    for output in thread.models[model]['output_files']:
        name = check_is_none(output, 'name')
        if name in results[0]:
            output["download_url"] = results[0][name]
            outputs.append(output)
    return outputs


def get_json(url):
    r = http.request('GET', url)
    data = json.loads(r.data.decode('utf-8'))
    r.release_conn()
    return data



import configparser
import os
import pathlib

__DEFAULT_MINT_API_CREDENTIALS_FILE__ = "~/.mint/credentials"

import click

DEFAULT_PROFILE = "default"


def get_credentials(profile):
    credentials_file = pathlib.Path(
        os.getenv("MINT_CREDENTIALS_FILE", __DEFAULT_MINT_API_CREDENTIALS_FILE__)
    ).expanduser()
    credentials = configparser.ConfigParser()
    if credentials_file.exists():
        credentials.read(credentials_file)
        if profile in credentials:
            return credentials[profile]
    elif credentials is None:
        return None
    elif credentials is not None and profile not in credentials and profile != ("%s" % DEFAULT_PROFILE):
        click.secho("WARNING: The profile doesn't exists. To configure it, run:\ndame configure -p {}".format(profile),
                    fg="yellow")
    return None


def configure_credentials(server, username, profile):
    credentials_file = pathlib.Path(
        os.getenv("MINT_CREDENTIALS_FILE", __DEFAULT_MINT_API_CREDENTIALS_FILE__)
    ).expanduser()
    os.makedirs(str(credentials_file.parent), exist_ok=True)

    credentials = configparser.ConfigParser()
    credentials.optionxform = str

    if credentials_file.exists():
        credentials.read(credentials_file)

    credentials[profile] = {
        "server": server,
        "username": username,
    }

    with credentials_file.open("w") as fh:
        credentials_file.parent.chmod(0o700)
        credentials_file.chmod(0o600)
        credentials.write(fh)



import os
import re
import stat
import uuid
from pathlib import Path

import click
import texttable as tt
from modelcatalog import ApiException, SampleResource

from dame._utils import log
from dame.executor import prepare_execution, get_engine, DOCKER_ENGINE, \
    run_docker, get_docker_cmd
from dame.local_file_manager import find_file_directory
from dame.modelcatalogapi import get_transformation_dataset
from dame.utils import create_yaml_from_resource, obtain_id
from dame.utils import url_validation

SCRIPT_FILENAME = "run"

data_set_property = ["id", "label"]
parameter_set_property = ["id", "label", "has_default_value"]


def show_model_configuration_details_dt(model_configuration):
    click.echo(click.style("Information about the model configuration", bold=True))
    if model_configuration and hasattr(model_configuration, "has_input") and getattr(model_configuration, "has_input"):
        click.echo(click.style("Inputs", bold=True))
        for _input in model_configuration.has_input:
            if hasattr(_input, "has_fixed_resource") and _input.has_fixed_resource and hasattr(
                    _input.has_fixed_resource[0], "value"):
                click.echo("- {}: {}".format(_input.label[0], _input.has_fixed_resource[0].value[0]))
            else:
                if hasattr(_input, "has_data_transformation"):
                    dts = _input.has_data_transformation
                    if dts:
                        for dt in dts:
                            label = getattr(dt, "label") if hasattr(dt, "label") else getattr(dt, "id")
                            click.echo(f"- {_input.label[0]}: {label[0]}")
                    else:
                        label = getattr(_input, "label") if hasattr(_input, "label") else getattr(_input, "id")
                        click.echo("- {}: {}".format(label[0], "No transformation"))


def show_model_configuration_details(model_configuration):
    click.echo(click.style("Information about the model configuration", bold=True))
    if model_configuration and hasattr(model_configuration, "has_input") and getattr(model_configuration, "has_input"):
        click.echo(click.style("Inputs", bold=True))
        for _input in model_configuration.has_input:
            if hasattr(_input, "has_fixed_resource") and _input.has_fixed_resource and hasattr(
                    _input.has_fixed_resource[0], "value"):
                click.echo("- {}: {}".format(_input.label[0], _input.has_fixed_resource[0].value[0]))
            else:
                label = getattr(_input, "label") if hasattr(_input, "label") else getattr(_input, "id")
                click.echo("- {}: {}".format(label[0], "No information"))
    if model_configuration and hasattr(model_configuration, "has_parameter") and getattr(model_configuration,
                                                                                         "has_parameter"):
        click.echo(click.style("Parameters", bold=True))
        for _parameter in model_configuration.has_parameter:
            short_value(_parameter, "has_default_value")
    if hasattr(model_configuration, "has_software_image") and getattr(model_configuration, "has_software_image"):
        try:
            click.echo(click.style("Docker Image", bold=True))
            image = getattr(model_configuration, "has_software_image")[0].label[0]
            click.echo("- {}: {} - https://hub.docker.com/r/{} ".format("Name", image, image.split(':')[0]))
        except AttributeError as e:
            raise AttributeError("No information available about the Docker Image.")
    else:
        raise AttributeError("No information available about the Docker Image.")
    if hasattr(model_configuration, "has_component_location") and getattr(model_configuration,
                                                                          "has_component_location"):
        try:
            click.echo(click.style("Component Location", bold=True))
            image = getattr(model_configuration, "has_component_location")[0]
            click.echo("- {}: {}".format("Link", image))
        except AttributeError as e:
            raise AttributeError("No information available about the executable component Location")
    else:
        raise AttributeError("No information available about the executable component Location")


def short_value(resource, prop):
    if hasattr(resource, prop) and getattr(resource, prop) is not None:
        value = getattr(resource, prop)
        click.echo("- {}: {}".format(getattr(resource, "label")[0], value[0]))
        return value[0]


def edit_parameters(model_configuration, interactive):
    if interactive:
        for index, _parameter in enumerate(model_configuration.has_parameter):
            value = short_value(_parameter, "has_default_value")
            new_value = click.prompt("Enter the new value", default=value)
            model_configuration.has_parameter[index].has_default_value = [new_value]
    return model_configuration


def print_disabled_data_option(data_dir):
    if not data_dir:
        click.secho("Search local files disabled. "
                    "You can enable it using  option -d or --data\n"
                    "$ dame run id -d example_directory", fg="blue")


def verify_input_parameters(model_configuration, interactive, data_dir, profile=None):
    profile_cmd = f"-p {profile}" if profile else ""
    if model_configuration.has_input is None:
        return []
    print_disabled_data_option(data_dir)
    for _input in model_configuration.has_input:
        uri = None
        if (not hasattr(_input, "has_fixed_resource") or _input.has_fixed_resource is None) and interactive:
            if hasattr(_input, "label") and hasattr(_input, "has_format"):
                extension = _input.has_format[0]
                extension = re.sub(r'^\.', '', extension)
                click.secho("To run this model configuration,"
                            "a {} file (.{} file) is required.".format(_input.label[0],
                                                                       extension),
                            fg="yellow")
            elif hasattr(_input, "label"):
                click.secho("To run this model configuration, a {} file is required.".format(_input.label[0]),
                            fg="yellow")
            else:
                click.secho("To run this model configuration, a {} file is required."
                            .format(_input.id), fg="yellow")
            if data_dir and hasattr(_input, "has_format"):
                click.secho("Searching local files in the {}\n"
                            "You can change the target directory using the option -d or --data")

                uri = find_file_directory(data_dir, extension)

            dataset_transformations = get_transformation_dataset(obtain_id(_input.id), profile=profile)
            if profile and dataset_transformations:
                click.secho("There are transformations available for this input. To use them, please exit and type", fg="blue")
                click.secho(f"$ dame transformation list {obtain_id(_input.id)}  {profile_cmd}", fg="blue")

            if uri is None:
                uri = click.prompt('Please enter a url')
                uri = uri.replace(" ", '')
                while not url_validation(uri):
                    uri = click.prompt('Please enter a url')

            create_sample_resource(_input, uri)
        elif not hasattr(_input, "has_fixed_resource") and not interactive:
            raise ValueError("Missing information")
    click.secho("The information needed to run the model is complete, and I can execute the model as follows:",
                fg="green")
    return model_configuration


def create_sample_resource(_input, uri):
    s = SampleResource(id="https://w3id.org/okn/i/mint/".format(str(uuid.uuid4())),
                       data_catalog_identifier="FFF-3s5c112e-c7ae-4cda-ba23-2e4f2286a18o",
                       value=[uri])
    _input.has_fixed_resource = [s.to_dict()]


def print_data_property_table(resource, property_selected={}):
    resource_dict = resource.to_dict()
    tab = tt.Texttable(max_width=100)
    headings = ['Property', 'Value']
    tab.header(headings)
    for key, value in resource_dict.items():
        if isinstance(value, dict) or key == "type" or key == "has_presentation":
            continue
        if property_selected:
            if key not in property_selected:
                continue
        tab.add_row([key, value])
    print(tab.draw())


def run_method_setup(setup, interactive, data_dir):
    """
    Call download_setup(): Download the setup(s) as yaml file
    Call execute_setup(): Read the yaml file and execute
    """
    try:
        name = obtain_id(setup.id)
        file_path = create_yaml_from_resource(resource=setup, name=name, output=Path('.'))
        component_src_dir, execution_dir, setup_cmd_line, setup_name, image = prepare_execution(file_path)

    except ApiException as e:
        click.secho("Unable to download the setup {}".format(e), fg="red")
        exit(1)

    try:
        execute_setups(component_src_dir, execution_dir, setup_cmd_line, setup_name, image, interactive)
        click.secho("[{}] The execution has been successful".format(setup_name), fg="green")
        click.secho("[{}] Results available at: {} ".format(setup_name, component_src_dir), fg="green")
    except Exception as e:
        log.error(e, exc_info=True)
        click.secho("[{}] The execution has failed".format(setup_name), fg="red")
        exit(1)


def execute_setups(component_src_dir, execution_dir, setup_cmd_line, setup_name, image, interactive):
    """
    Find the setup files if the path is a directory and execute it
    """
    try:
        engine = get_engine()
        click.secho(f"Engine: {engine}", fg="blue")
    except Exception as e:
        click.secho("Docker is not running or installed".format(e), fg="red")
        exit(1)

    os.chmod(component_src_dir / SCRIPT_FILENAME, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
    if engine == DOCKER_ENGINE:
        try:
            mint_volumes = {str(Path(component_src_dir).absolute()): {'bind': '/tmp/mint', 'mode': 'rw'}}
            docker_cmd_pretty = get_docker_cmd(image, setup_cmd_line, mint_volumes)
            show_execution_info(component_src_dir, interactive, docker_cmd_pretty)
            run_docker(setup_cmd_line, execution_dir, component_src_dir, setup_name, image, mint_volumes)
        except Exception as e:
            raise e


def show_execution_info(component_src_dir, interactive, singularity_cmd_pretty):
    click.echo("Invocation command \ncd {}\n{}".format(component_src_dir, singularity_cmd_pretty))
    if interactive and not click.confirm("Do you want to proceed and submit it for execution?", default=True):
        exit(0)


def find_setup_files(path):
    setup_files = []
    if os.path.isdir(path):
        path = Path(path)
        for file in os.listdir(path):
            if file.endswith(".yaml") or file.endswith(".yml"):
                setup_files.append(path / file)
    elif os.path.isfile(path):
        default_path = Path('.')
        setup_files.append(default_path / path)
    return setup_files


def print_table_list(items):
    headings = ['Id', 'Description']
    tab = tt.Texttable()
    tab.header(headings)
    for item in items:
        _id = obtain_id(item.id)
        _label = "".join(item.label) if hasattr(item, "label") and getattr(item, "label") else "No information"
        _description = "".join(item.description) if hasattr(item, "description") and getattr(item,
                                                                                             "description") else _label
        tab.add_row([_id, _description])
    print(tab.draw())


def print_table_list_data(items):
    headings = ['Id', 'Description']
    tab = tt.Texttable(max_width=90)
    tab.header(headings)
    for item in items:
        _id = item["id"]
        _description = "".join(item["description"]) if "description" in item and item[
            "description"] else "No information"
        tab.add_row([_id, _description])
    print(tab.draw())



from __future__ import print_function

import click
import modelcatalog
from modelcatalog import ApiClient
from modelcatalog.rest import ApiException

from dame.configuration import get_credentials, DEFAULT_PROFILE

USERNAME = "mint@isi.edu"


def api_configuration(profile):
    credentials = get_credentials(profile)
    if credentials is None:
        return ApiClient(), USERNAME
    configuration = modelcatalog.Configuration()
    package_version = configuration.host.split('/')[-1].replace("v", '')
    configuration_version = credentials["server"].split('/')[-1].replace("v", '')
    if package_version > configuration_version:
        click.secho(
            f"""WARNING: Your credentials are using Model Catalog version {configuration_version},
            but the version {package_version} is available.
            You should consider upgrading via the 'dame configure -p {profile}'""",
            fg="yellow",
        )
        click.secho("DAME is going to use the newest version", fg="yellow")
    else:
        configuration.host = credentials["server"]
    return ApiClient(configuration=configuration), credentials["username"]


def list_model_configuration(label=None, profile=DEFAULT_PROFILE):
    api, username = api_configuration(profile)
    api_instance = modelcatalog.ModelConfigurationApi(api)
    try:
        api_response = api_instance.modelconfigurations_get(username=username)
        return api_response
    except ApiException as e:
        raise e


def get_data_transformation(profile=DEFAULT_PROFILE):
    api, username = api_configuration(profile)
    api_instance = modelcatalog.DataTransformationApi(api)
    try:
        api_response = api_instance.datatransformations_get(username=username)
        return api_response
    except ApiException as e:
        raise e


def get_model_configuration(_id, profile=DEFAULT_PROFILE):
    api, username = api_configuration(profile)
    api_instance = modelcatalog.ModelConfigurationApi(api)
    try:
        api_response = api_instance.custom_modelconfigurations_id_get(_id, username=username)
        return api_response
    except ApiException as e:
        raise e


def get_setup(_id, profile=DEFAULT_PROFILE):
    api, username = api_configuration(profile)
    api_instance = modelcatalog.ModelConfigurationSetupApi(api)
    try:
        # Get a ModelConfiguration
        api_response = api_instance.custom_modelconfigurationsetups_id_get(_id, username=username)
        return api_response
    except ApiException as e:
        raise e


def list_setup(label=None, profile=DEFAULT_PROFILE):
    api, username = api_configuration(profile)
    api_instance = modelcatalog.ModelConfigurationSetupApi(api)
    try:
        # Get a ModelConfigurationSetup
        api_response = api_instance.modelconfigurationsetups_get(username=username)
        return api_response
    except ApiException as e:
        raise e


def get_transformation_dataset(data_specification_id, profile=DEFAULT_PROFILE):
    # Create an instance of the API class

    items = []
    return items;
    #FIXME: I do no find this query on the server.

    api, username = api_configuration(profile)
    api_instance = modelcatalog.DataTransformationApi(api)
    custom_query_name = 'custom_datatransformations'  # str | Name of the custom query (optional) (default to 'custom_datatransformations')

    try:
        # Gets a list of data transformations related a dataset
        api_response = api_instance.\
                                    custom_dataspecifications_id_datatransformations_get(
                                        data_specification_id,
                                        custom_query_name=custom_query_name,
                                        username=username)

        for i in api_response:
            if i.id:
                items.append(i)


        return items
    except ApiException as e:
        raise e




import os
from setuptools import find_packages, setup

def find_package_data(dirname):
    def find_paths(dirname):
        items = []
        for fname in os.listdir(dirname):
            path = os.path.join(dirname, fname)
            if os.path.isdir(path):
                items += find_paths(path)
            elif not path.endswith(".py") and not path.endswith(".pyc"):
                items.append(path)
        return items

    items = find_paths(dirname)
    return [os.path.relpath(path, dirname) for path in items]

# Utility function to read the README file.
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), encoding='utf-8').read()

install_requires = [
    "Click>=7.0",
    "modelcatalog-api==7.1.0",
    "texttable>=1.6.2",
    "validators>=0.14.1",
    "ingestion-api>=1.3.3",
    "PyYAML>=5.1.2",
    "semver>=2.8.1",
    "requests",
    "jsonschema>=3.0.0",
    "configparser>=5.0.0"
    "certifi>=2019.11.28",
    "texttable",
    "docker>=4.2.0"
]

version = {}
with open("src/dame/__init__.py") as fp:
    exec(fp.read(), version)



# This call to setup_name() does all the work
setup(
    name="dame-cli",
    version=version["__version__"],
    description="A execution manager cli for execution",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/mintproject/dame_cli",
    author="Maximiliano Osorio",
    author_email="mosorio@isi.edu",
    license="Apache-2",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
    ],
    entry_points={"console_scripts": ["dame = dame.__main__:cli"]},
    package_dir={"": "src"},
    packages=find_packages(where="src", exclude=["dame.tests*"]),
    package_data={"dame": find_package_data("src/dame")},
    exclude_package_data={"dame": ["tests/*"]},
    zip_safe=False,
    install_requires=install_requires,
    python_requires=">=3.6.0",
)



import os
import tempfile
from pathlib import Path
from zipfile import ZipFile

import click
import requests
import validators
import yaml

DOC_LINK = "https://dame-cli.readthedocs.io/en/latest/"
ignore_dirs = ["__MACOSX"]
SERVER = "https://dev.mint.isi.edu"
DATA_DIR = "./data"


def convert_object_to_dict(o):
    if isinstance(o, object):
        return o.to_dict()
    return o


def check_is_none(item, key):
    item[key] if key in item else ''


def create_yaml_from_resource(resource, name, output):
    filename = name + ".yaml"
    path = Path.cwd() / output / filename
    with open(path, mode='w+') as fid:
        yaml.dump(resource, fid)
    return path


def url_validation(url):
    if validators.url(url):
        return True
    elif Path.is_file(Path(url).expanduser().resolve()):
        return True
    click.secho("URL is not valid or the file doesn't exists.", fg="red")
    return False


def obtain_id(url):
    if validators.url(url):
        return url.split('/')[-1]
    return url


def download_extract_zip(url, _dir):
    temp = tempfile.NamedTemporaryFile(mode="wb", prefix="component_", delete=False)
    content = download_file(url)
    temp.write(content)
    temp.close()
    with ZipFile(temp.name, 'r') as zip_file:
        zip_file.extractall(_dir)
    directories = os.listdir(_dir)
    if isinstance(directories, list):
        try:
            for ignore_dir in ignore_dirs:
                directories.remove(ignore_dir)
        except:
            pass

    if len(directories) == 1:
        return os.path.join(_dir, directories[0])
    else:
        raise ValueError("The zipfile must has one directory.")


def download_file(url):
    headers = {'Cache-Control': 'no-cache'}
    r = requests.get(url, allow_redirects=True, headers=headers)
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError:
        raise requests.exceptions.HTTPError(r)
    except requests.exceptions.RequestException:
        raise requests.exceptions.RequestException(r)
    return r.content


def validate_suffix(suffix):
    return suffix if suffix.startswith('.') else ".{}".format(suffix)


def download_data_file(url, _dir, format):
    headers = {'Cache-Control': 'no-cache'}
    r = requests.get(url, allow_redirects=True, headers=headers)
    filename = url.split('/')[-1]
    filepath = Path(_dir / filename)
    if filepath.suffix == "" and format:
        filepath = filepath.with_suffix(validate_suffix(format))
    with requests.get(url, stream=True, headers=headers) as r:
        r.raise_for_status()
        with open(filepath, 'wb') as f:
            for chunk in r.iter_content(chunk_size=8192):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
    return filepath, filename



import shutil
import uuid
from pathlib import Path
import docker
import validators
from yaml import load, Loader
from dame.utils import download_extract_zip, obtain_id, convert_object_to_dict, download_data_file

KEYS_REQUIRED_PARAMETER = {"has_default_value", "position"}
KEYS_REQUIRED_OUTPUT = {"label", "has_format", "position"}
KEYS_REQUIRED_INPUT = {"has_fixed_resource"}

EXECUTION_DIRECTORY = "executions"

DOCKER_ENGINE = "DOCKER"


def get_engine():
    try:
        client = docker.from_env()
        client.info()
        return DOCKER_ENGINE
    except Exception as e:
        raise e


def get_file(destination_dir, url, _format):
    """
    Get the files from a url or the
    :return: The filename of the file. Must be the filename
    :rtype: str
    :param _format:
    :type _format:  str
    :param url:
    :type url: str
    :param destination_dir: The destination directory
    :type destination_dir: Path
    """
    if validators.url(url):
        file_path, file_name = download_data_file(url, destination_dir, _format)
        return file_path.name
    elif Path(url).is_file():
        return Path(shutil.copy(str(Path(url)), str(destination_dir))).name


def build_input(inputs, destination_dir):
    """
    Download or search the file. Loop the inputs (metadata) of Model Configuration or Model Configuration Setup
    :return:
    :rtype:
    :param inputs: A dictionary following DataSpecificationFile
    :type inputs: dict
    :param destination_dir: The destination directory
    :type destination_dir: Path
    :return: The cmd_line related to the input -i1 file1 -i2 file2
    :param data_dir: The local directory where the files are
    :type data_dir: Path
    :rtype: str
    """
    line = ""
    for _input in inputs:
        _input = convert_object_to_dict(_input)
        if not _input.keys() >= KEYS_REQUIRED_INPUT or _input['has_fixed_resource'] is None:
            raise ValueError(f'{_input["id"]} has not a fixedResource')
        else:
            url = _input["has_fixed_resource"][0]["value"][0]
            if "has_format" in _input:
                _format = _input["has_fixed_resource"][0]["value"][0]
            else:
                _format = None
            file_name = get_file(destination_dir, url, _format)
            position = _input["position"][0]
            line += " -i{} {}".format(position, file_name)
    return line


def build_output(outputs):
    line = ""
    for _output in outputs:
        _output = convert_object_to_dict(_output)
        if not _output.keys() >= KEYS_REQUIRED_OUTPUT:
            raise ValueError(f'{_output["id"]}  has not the required information')
        label = _output["label"][0]
        extension = _output["has_format"][0]
        position = _output["position"][0]
        line += " -o{} {}.{}".format(position, label, extension)
    return line


def build_parameter(parameters):
    line = ""
    for _parameter in parameters:
        _parameter = convert_object_to_dict(_parameter)
        if not _parameter.keys() >= KEYS_REQUIRED_PARAMETER:
            raise ValueError(f'{_parameter["id"]} has not the required information ')
        if "has_fixed_resource" in _parameter:
            value = _parameter["has_fixed_resource"][0]
        else:
            value = _parameter["has_default_value"][0]
        position = _parameter["position"][0]
        if 'has_data_type' not in  _parameter or _parameter["has_data_type"] is None or "string" in _parameter["has_data_type"]:
            line += " -p{} \"{}\"".format(position, value)
        else:
            line += " -p{} {}".format(position, value)

    return line


def build_command_line(resource, _dir):
    line = './run '
    setup_name = obtain_id(resource.id)
    inputs = resource.has_input
    try:
        parameters = resource.has_parameter
    except:
        parameters = None
    outputs = resource.has_output
    component_url = resource.has_component_location[0]
    software_image = resource.has_software_image
    if software_image:
        has_software_image = software_image[0].label[0]
    else:
        raise ValueError("Software is not available")
    component_dir = download_extract_zip(component_url, _dir)
    path = Path(component_dir)
    src_path = path / "src"
    if inputs:
        l = build_input(inputs, src_path)
        line += " {}".format(l)
    if outputs:
        l = build_output(outputs)
        line += " {}".format(l)
    if parameters is not None:
        l = build_parameter(parameters)
        line += " {}".format(l)
    return has_software_image, line, src_path


def prepare_execution(setup_path):
    """
    Create the execution directory and call the method to get the inputs and execution line
    :param setup_path: the path of the YAML file
    :type setup_path: Path
    :return:
        - src_path: (:py:class:`Path`) - Path component source directory
        - execution_dir: (:py:class:`Path`) - Path execution directory
        - setup_cmd_line: (:py:class:`Path`) - The execution line (./run)
        - setup_name: (:py:class:`Path`) - The name of setup
        - image: (:py:class:`Path`) - The name of the image
    """
    _dir = Path("%s/" % EXECUTION_DIRECTORY)
    _dir.mkdir(parents=True, exist_ok=True)
    setup_dict = load(setup_path.open(), Loader=Loader)
    setup_name = obtain_id(setup_dict.id)
    execution_dir = "{}/{}_{}".format(_dir, setup_name, uuid.uuid1())
    execution_dir_path = Path(execution_dir)
    execution_dir_path.mkdir(parents=True, exist_ok=True)
    try:
        image, setup_cmd_line, src_path = build_command_line(setup_dict, execution_dir_path)
    except Exception as e:
        raise e
    return src_path, execution_dir, setup_cmd_line, setup_name, image


def docker_pull(client, image):
    client = docker.APIClient()
    try:
        result_itr = client.pull(image, stream=True, decode=True)
    except docker.errors.APIError as ex:
        raise docker.errors.APIError

    for chunk in result_itr:
        try:
            if "progressDetail" in chunk:
                print("{}: Downloading {}".format(chunk['id'], chunk['progress']))
        except:
            pass


def run_docker(component_cmd, execution_dir, component_dir, setup_name, image, volumes):
    log_file_path = "{}/output.log".format(execution_dir)
    client = docker.from_env()
    docker_pull(client, image)
    res = client.containers.run(command=component_cmd,
                                image=image,
                                volumes=volumes,
                                entrypoint='/bin/bash',
                                working_dir='/tmp/mint',
                                detach=True,
                                stream=True,
                                remove=True
                                )
    with open(log_file_path, 'wb') as log_file:
        for chunk in res.logs(stream=True):
            print(chunk.decode('utf-8'))
            log_file.write(chunk)



def get_docker_cmd(image, setup_cmd_line, mint_volumes):
    volume_line = ""
    for volume in mint_volumes.keys():
        volume_line += "-v {}:{}".format(volume, mint_volumes[volume]["bind"])
    volume_line += " -w {}".format(mint_volumes[volume]["bind"])

    return "docker run -ti {} {} {}".format(volume_line, image, setup_cmd_line)



from pathlib import Path

import click


def find_file_directory(data_dir, _format=None):
    """
    Find the files by format on the data_dir
    :param data_dir: The local directory where the files are
    :type data_dir: Path
    :param _format:
    :type _format:
    """
    if _format is not None:
        files = [f for f in data_dir.glob("*{}".format(_format))]
        if len(files) > 1:
            print_choices([f.resolve().expanduser() for f in files])
            file_index = click.prompt("Select the file",
                                      type=click.Choice(range(1, len(files) + 1)),
                                      show_choices=True,
                                      value_proc=parse
                                      )
            click.secho("Selected from your computer {}".format(files[file_index-1]))
            return str(files[file_index-1])
        elif len(files) == 1:
            click.secho("Selected from your computer {}".format(files[0]))
            return str(files[0])
        else:
            click.secho("There is not files with format {} in the directory {}".format(_format, data_dir))
    return None

def print_choices(choices):
    for index, choice in enumerate(choices):
        click.echo("[{}] {}".format(index + 1, choice))


def parse(value):
    try:
        return int(value)
    except:
        return value
